@@ -26,6 +26,7 @@ from openstack import exceptions
 from openstack.network.v2._proxy import Proxy
 from openstack import proxy
 from openstack import utils
+from openstack import warnings as os_warnings
 
 _CONFIG_DOC_URL = (
     "https://docs.openstack.org/openstacksdk/latest/"
@@ -66,8 +67,8 @@ class FloatingIPCloudMixin:
         # `filters` could be a jmespath expression which Neutron server doesn't
         # understand, obviously.
         warnings.warn(
-            "search_floating_ips is deprecated. "
-            "Use search_resource instead."
+            "search_floating_ips is deprecated. Use search_resource instead.",
+            os_warnings.OpenStackDeprecationWarning,
         )
         if self._use_neutron_floating() and isinstance(filters, dict):
             return list(self.network.ips(**filters))
@@ -318,7 +319,7 @@ class FloatingIPCloudMixin:
             is not specified and cannot be found.
         """
 
-        with _utils.shade_exceptions(
+        with _utils.openstacksdk_exceptions(
             "Unable to create floating IP in pool {pool}".format(pool=pool)
         ):
             if pool is None:
@@ -535,7 +536,7 @@ class FloatingIPCloudMixin:
                 try:
                     for count in utils.iterate_timeout(
                         timeout,
-                        "Timeout waiting for the floating IP" " to be ACTIVE",
+                        "Timeout waiting for the floating IP to be ACTIVE",
                         wait=self._FLOAT_AGE,
                     ):
                         fip = self.get_floating_ip(fip_id)
@@ -575,7 +576,7 @@ class FloatingIPCloudMixin:
         return fip
 
     def _nova_create_floating_ip(self, pool=None):
-        with _utils.shade_exceptions(
+        with _utils.openstacksdk_exceptions(
             "Unable to create floating IP in pool {pool}".format(pool=pool)
         ):
             if pool is None:
