@@ -176,6 +176,7 @@ try to find it and if that fails, you would create it::
 Additional information about the services can be found in the
 :ref:`service-proxies` documentation.
 """
+import atexit
 import concurrent.futures
 import warnings
 import weakref
@@ -194,7 +195,6 @@ from openstack import _services_mixin
 from openstack.cloud import _accelerator
 from openstack.cloud import _baremetal
 from openstack.cloud import _block_storage
-from openstack.cloud import _clustering
 from openstack.cloud import _coe
 from openstack.cloud import _compute
 from openstack.cloud import _dns
@@ -263,7 +263,6 @@ class Connection(
     _baremetal.BaremetalCloudMixin,
     _block_storage.BlockStorageCloudMixin,
     _compute.ComputeCloudMixin,
-    _clustering.ClusteringCloudMixin,
     _coe.CoeCloudMixin,
     _dns.DnsCloudMixin,
     _floating_ip.FloatingIPCloudMixin,
@@ -419,7 +418,6 @@ class Connection(
         _accelerator.AcceleratorCloudMixin.__init__(self)
         _baremetal.BaremetalCloudMixin.__init__(self)
         _block_storage.BlockStorageCloudMixin.__init__(self)
-        _clustering.ClusteringCloudMixin.__init__(self)
         _coe.CoeCloudMixin.__init__(self)
         _compute.ComputeCloudMixin.__init__(self)
         _dns.DnsCloudMixin.__init__(self)
@@ -475,9 +473,8 @@ class Connection(
                 'additional_metric_tags'
             ] = self.config.config['additional_metric_tags']
 
-    def __del__(self):
-        # try to force release of resources and save authorization
-        self.close()
+        # Register cleanup steps
+        atexit.register(self.close)
 
     @property
     def session(self):
@@ -554,9 +551,9 @@ class Connection(
 
     def close(self):
         """Release any resources held open."""
+        self.config.set_auth_cache()
         if self.__pool_executor:
             self.__pool_executor.shutdown()
-        self.config.set_auth_cache()
 
     def set_global_request_id(self, global_request_id):
         self._global_request_id = global_request_id
