@@ -13,28 +13,27 @@
 from openstack import proxy
 from openstack import resource
 from openstack.shared_file_system.v2 import (
-    availability_zone as _availability_zone)
-from openstack.shared_file_system.v2 import (
-    share_export_locations as _share_export_locations
-)
-from openstack.shared_file_system.v2 import (
-    share_network as _share_network
-)
-from openstack.shared_file_system.v2 import (
-    share_snapshot as _share_snapshot
-)
-from openstack.shared_file_system.v2 import (
-    share_snapshot_instance as _share_snapshot_instance
-)
-from openstack.shared_file_system.v2 import (
-    storage_pool as _storage_pool
-)
-from openstack.shared_file_system.v2 import (
-    user_message as _user_message
+    availability_zone as _availability_zone,
 )
 from openstack.shared_file_system.v2 import limit as _limit
 from openstack.shared_file_system.v2 import share as _share
+from openstack.shared_file_system.v2 import (
+    share_access_rule as _share_access_rule,
+)
+from openstack.shared_file_system.v2 import (
+    share_export_locations as _share_export_locations,
+)
 from openstack.shared_file_system.v2 import share_instance as _share_instance
+from openstack.shared_file_system.v2 import share_network as _share_network
+from openstack.shared_file_system.v2 import (
+    share_network_subnet as _share_network_subnet,
+)
+from openstack.shared_file_system.v2 import share_snapshot as _share_snapshot
+from openstack.shared_file_system.v2 import (
+    share_snapshot_instance as _share_snapshot_instance,
+)
+from openstack.shared_file_system.v2 import storage_pool as _storage_pool
+from openstack.shared_file_system.v2 import user_message as _user_message
 
 
 class Proxy(proxy.Proxy):
@@ -46,11 +45,11 @@ class Proxy(proxy.Proxy):
         "limit": _limit.Limit,
         "share": _share.Share,
         "share_network": _share_network.ShareNetwork,
-        "share_snapshot_instance":
-            _share_snapshot_instance.ShareSnapshotInstance,
+        "share_network_subnet": _share_network_subnet.ShareNetworkSubnet,
+        "share_snapshot_instance": _share_snapshot_instance.ShareSnapshotInstance,  # noqa: E501
         "share_instance": _share_instance.ShareInstance,
-        "share_export_locations":
-            _share_export_locations.ShareExportLocation,
+        "share_export_locations": _share_export_locations.ShareExportLocation,
+        "share_access_rule": _share_access_rule.ShareAccessRule,
     }
 
     def availability_zones(self):
@@ -110,6 +109,26 @@ class Proxy(proxy.Proxy):
         base_path = '/shares/detail' if details else None
         return self._list(_share.Share, base_path=base_path, **query)
 
+    def find_share(self, name_or_id, ignore_missing=True, **query):
+        """Find a single share
+
+        :param name_or_id: The name or ID of a share.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the resource does not exist.
+            When set to ``True``, None will be returned when
+            attempting to find a nonexistent resource.
+        :param dict query: Any additional parameters to be passed into
+            underlying methods. such as query filters.
+
+        :returns: One :class:`~openstack.shared_file_system.v2.share.Share`
+                  or None
+        """
+
+        return self._find(
+            _share.Share, name_or_id, ignore_missing=ignore_missing, **query
+        )
+
     def get_share(self, share_id):
         """Lists details of a single share
 
@@ -126,8 +145,7 @@ class Proxy(proxy.Proxy):
         :returns: Result of the ``delete``
         :rtype: ``None``
         """
-        self._delete(_share.Share, share,
-                     ignore_missing=ignore_missing)
+        self._delete(_share.Share, share, ignore_missing=ignore_missing)
 
     def update_share(self, share_id, **attrs):
         """Updates details of a single share.
@@ -164,12 +182,7 @@ class Proxy(proxy.Proxy):
         res.revert_to_snapshot(self, snapshot_id)
 
     def resize_share(
-        self,
-        share_id,
-        new_size,
-        no_shrink=False,
-        no_extend=False,
-        force=False
+        self, share_id, new_size, no_shrink=False, no_extend=False, force=False
     ):
         """Resizes a share, extending/shrinking the share as needed.
 
@@ -198,8 +211,9 @@ class Proxy(proxy.Proxy):
         elif new_size < res.size and no_shrink is not True:
             res.shrink_share(self, new_size)
 
-    def wait_for_status(self, res, status='active', failures=None,
-                        interval=2, wait=120):
+    def wait_for_status(
+        self, res, status='active', failures=None, interval=2, wait=120
+    ):
         """Wait for a resource to be in a particular status.
         :param res: The resource to wait on to reach the specified status.
             The resource must have a ``status`` attribute.
@@ -221,7 +235,8 @@ class Proxy(proxy.Proxy):
         """
         failures = [] if failures is None else failures
         return resource.wait_for_status(
-            self, res, status, failures, interval, wait)
+            self, res, status, failures, interval, wait
+        )
 
     def storage_pools(self, details=True, **query):
         """Lists all back-end storage pools with details
@@ -240,7 +255,8 @@ class Proxy(proxy.Proxy):
         """
         base_path = '/scheduler-stats/pools/detail' if details else None
         return self._list(
-            _storage_pool.StoragePool, base_path=base_path, **query)
+            _storage_pool.StoragePool, base_path=base_path, **query
+        )
 
     def user_messages(self, **query):
         """List shared file system user messages
@@ -270,8 +286,7 @@ class Proxy(proxy.Proxy):
         :rtype:
             :class:`~openstack.shared_file_system.v2.user_message.UserMessage`
         """
-        return self._list(
-            _user_message.UserMessage, **query)
+        return self._list(_user_message.UserMessage, **query)
 
     def get_user_message(self, message_id):
         """List details of a single user message
@@ -292,8 +307,10 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.shared_file_system.v2.user_message.UserMessage`
         """
         return self._delete(
-            _user_message.UserMessage, message_id,
-            ignore_missing=ignore_missing)
+            _user_message.UserMessage,
+            message_id,
+            ignore_missing=ignore_missing,
+        )
 
     def limits(self, **query):
         """Lists all share limits.
@@ -304,8 +321,7 @@ class Proxy(proxy.Proxy):
         :returns: A generator of manila share limits resources
         :rtype: :class:`~openstack.shared_file_system.v2.limit.Limit`
         """
-        return self._list(
-            _limit.Limit, **query)
+        return self._list(_limit.Limit, **query)
 
     def share_snapshots(self, details=True, **query):
         """Lists all share snapshots with details.
@@ -316,20 +332,21 @@ class Proxy(proxy.Proxy):
             * project_id: The ID of the user or service making the API request.
 
         :returns: A generator of manila share snapshot resources
-        :rtype: :class:`~openstack.shared_file_system.v2.
-            share_snapshot.ShareSnapshot`
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_snapshot.ShareSnapshot`
         """
         base_path = '/snapshots/detail' if details else None
         return self._list(
-            _share_snapshot.ShareSnapshot, base_path=base_path, **query)
+            _share_snapshot.ShareSnapshot, base_path=base_path, **query
+        )
 
     def get_share_snapshot(self, snapshot_id):
         """Lists details of a single share snapshot
 
         :param snapshot_id: The ID of the snapshot to get
         :returns: Details of the identified share snapshot
-        :rtype: :class:`~openstack.shared_file_system.v2.
-            share_snapshot.ShareSnapshot`
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_snapshot.ShareSnapshot`
         """
         return self._get(_share_snapshot.ShareSnapshot, snapshot_id)
 
@@ -337,8 +354,8 @@ class Proxy(proxy.Proxy):
         """Creates a share snapshot from attributes
 
         :returns: Details of the new share snapshot
-        :rtype: :class:`~openstack.shared_file_system.v2.
-            share_snapshot.ShareSnapshot`
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_snapshot.ShareSnapshot`
         """
         return self._create(_share_snapshot.ShareSnapshot, **attrs)
 
@@ -348,11 +365,12 @@ class Proxy(proxy.Proxy):
         :param snapshot_id: The ID of the snapshot to update
         :pram dict attrs: The attributes to update on the snapshot
         :returns: the updated share snapshot
-        :rtype: :class:`~openstack.shared_file_system.v2.
-            share_snapshot.ShareSnapshot`
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_snapshot.ShareSnapshot`
         """
-        return self._update(_share_snapshot.ShareSnapshot, snapshot_id,
-                            **attrs)
+        return self._update(
+            _share_snapshot.ShareSnapshot, snapshot_id, **attrs
+        )
 
     def delete_share_snapshot(self, snapshot_id, ignore_missing=True):
         """Deletes a single share snapshot
@@ -361,11 +379,89 @@ class Proxy(proxy.Proxy):
         :returns: Result of the ``delete``
         :rtype: ``None``
         """
-        self._delete(_share_snapshot.ShareSnapshot, snapshot_id,
-                     ignore_missing=ignore_missing)
+        self._delete(
+            _share_snapshot.ShareSnapshot,
+            snapshot_id,
+            ignore_missing=ignore_missing,
+        )
+
+    # ========= Network Subnets ==========
+    def share_network_subnets(self, share_network_id):
+        """Lists all share network subnets with details.
+
+        :param share_network_id: The id of the share network for which
+            Share Network Subnets should be listed.
+        :returns: A generator of manila share network subnets
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_network_subnet.ShareNetworkSubnet`
+        """
+        return self._list(
+            _share_network_subnet.ShareNetworkSubnet,
+            share_network_id=share_network_id,
+        )
+
+    def get_share_network_subnet(
+        self,
+        share_network_id,
+        share_network_subnet_id,
+    ):
+        """Lists details of a single share network subnet.
+
+        :param share_network_id: The id of the share network associated
+            with the Share Network Subnet.
+        :param share_network_subnet_id: The id of the Share Network Subnet
+            to retrieve.
+        :returns: Details of the identified share network subnet
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_network_subnet.ShareNetworkSubnet`
+        """
+
+        return self._get(
+            _share_network_subnet.ShareNetworkSubnet,
+            share_network_subnet_id,
+            share_network_id=share_network_id,
+        )
+
+    def create_share_network_subnet(self, share_network_id, **attrs):
+        """Creates a share network subnet from attributes
+
+        :param share_network_id: The id of the share network wthin which the
+            the Share Network Subnet should be created.
+        :param dict attrs: Attributes which will be used to create
+            a share network subnet.
+        :returns: Details of the new share network subnet.
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share_network_subnet.ShareNetworkSubnet`
+        """
+        return self._create(
+            _share_network_subnet.ShareNetworkSubnet,
+            **attrs,
+            share_network_id=share_network_id
+        )
+
+    def delete_share_network_subnet(
+        self, share_network_id, share_network_subnet, ignore_missing=True
+    ):
+        """Deletes a share network subnet.
+
+        :param share_network_id: The id of the Share Network associated with
+            the Share Network Subnet.
+        :param share_network_subnet: The id of the Share Network Subnet
+            which should be deleted.
+        :returns: Result of the ``delete``
+        :rtype: None
+        """
+
+        self._delete(
+            _share_network_subnet.ShareNetworkSubnet,
+            share_network_subnet,
+            share_network_id=share_network_id,
+            ignore_missing=ignore_missing,
+        )
 
     def wait_for_delete(self, res, interval=2, wait=120):
         """Wait for a resource to be deleted.
+
         :param res: The resource to wait on to be deleted.
         :type resource: A :class:`~openstack.resource.Resource` object.
         :param interval: Number of seconds to wait before to consecutive
@@ -374,7 +470,7 @@ class Proxy(proxy.Proxy):
             Default to 120.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-                 to delete failed to occur in the specified seconds.
+            to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait)
 
@@ -397,8 +493,11 @@ class Proxy(proxy.Proxy):
             share_snapshot_instance.ShareSnapshotInstance`
         """
         base_path = '/snapshot-instances/detail' if details else None
-        return self._list(_share_snapshot_instance.ShareSnapshotInstance,
-                          base_path=base_path, **query)
+        return self._list(
+            _share_snapshot_instance.ShareSnapshotInstance,
+            base_path=base_path,
+            **query
+        )
 
     def get_share_snapshot_instance(self, snapshot_instance_id):
         """Lists details of a single share snapshot instance
@@ -408,8 +507,10 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.shared_file_system.v2.
             share_snapshot_instance.ShareSnapshotInstance`
         """
-        return self._get(_share_snapshot_instance.ShareSnapshotInstance,
-                         snapshot_instance_id)
+        return self._get(
+            _share_snapshot_instance.ShareSnapshotInstance,
+            snapshot_instance_id,
+        )
 
     def share_networks(self, details=True, **query):
         """Lists all share networks with details.
@@ -431,7 +532,8 @@ class Proxy(proxy.Proxy):
         """
         base_path = '/share-networks/detail' if details else None
         return self._list(
-            _share_network.ShareNetwork, base_path=base_path, **query)
+            _share_network.ShareNetwork, base_path=base_path, **query
+        )
 
     def get_share_network(self, share_network_id):
         """Lists details of a single share network
@@ -450,8 +552,10 @@ class Proxy(proxy.Proxy):
         :rtype: ``None``
         """
         self._delete(
-            _share_network.ShareNetwork, share_network_id,
-            ignore_missing=ignore_missing)
+            _share_network.ShareNetwork,
+            share_network_id,
+            ignore_missing=ignore_missing,
+        )
 
     def update_share_network(self, share_network_id, **attrs):
         """Updates details of a single share network.
@@ -463,7 +567,8 @@ class Proxy(proxy.Proxy):
             share_network.ShareNetwork`
         """
         return self._update(
-            _share_network.ShareNetwork, share_network_id, **attrs)
+            _share_network.ShareNetwork, share_network_id, **attrs
+        )
 
     def create_share_network(self, **attrs):
         """Creates a share network from attributes
@@ -493,8 +598,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.shared_file_system.v2.
             share_instance.ShareInstance`
         """
-        return self._list(
-            _share_instance.ShareInstance, **query)
+        return self._list(_share_instance.ShareInstance, **query)
 
     def get_share_instance(self, share_instance_id):
         """Shows details for a single share instance
@@ -515,8 +619,9 @@ class Proxy(proxy.Proxy):
 
         :returns: ``None``
         """
-        res = self._get_resource(_share_instance.ShareInstance,
-                                 share_instance_id)
+        res = self._get_resource(
+            _share_instance.ShareInstance, share_instance_id
+        )
         res.reset_status(self, status)
 
     def delete_share_instance(self, share_instance_id):
@@ -526,8 +631,9 @@ class Proxy(proxy.Proxy):
 
         :returns: ``None``
         """
-        res = self._get_resource(_share_instance.ShareInstance,
-                                 share_instance_id)
+        res = self._get_resource(
+            _share_instance.ShareInstance, share_instance_id
+        )
         res.force_delete(self)
 
     def export_locations(self, share_id):
@@ -538,8 +644,9 @@ class Proxy(proxy.Proxy):
         :rtype: List of :class:`~openstack.shared_filesystem_storage.v2.
             share_export_locations.ShareExportLocations`
         """
-        return self._list(_share_export_locations.ShareExportLocation,
-                          share_id=share_id)
+        return self._list(
+            _share_export_locations.ShareExportLocation, share_id=share_id
+        )
 
     def get_export_location(self, export_location, share_id):
         """List details of export location
@@ -554,4 +661,56 @@ class Proxy(proxy.Proxy):
         export_location_id = resource.Resource._get_id(export_location)
         return self._get(
             _share_export_locations.ShareExportLocation,
-            export_location_id, share_id=share_id)
+            export_location_id,
+            share_id=share_id,
+        )
+
+    def access_rules(self, share, **query):
+        """Lists the access rules on a share.
+
+        :returns: A generator of the share access rules.
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_access_rules.ShareAccessRules`
+        """
+        share = self._get_resource(_share.Share, share)
+        return self._list(
+            _share_access_rule.ShareAccessRule, share_id=share.id, **query
+        )
+
+    def get_access_rule(self, access_id):
+        """List details of an access rule.
+
+        :param access_id: The id of the access rule to get
+        :returns: Details of the identified access rule.
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_access_rules.ShareAccessRules`
+        """
+        return self._get(_share_access_rule.ShareAccessRule, access_id)
+
+    def create_access_rule(self, share_id, **attrs):
+        """Creates an access rule from attributes
+
+        :returns: Details of the new access rule
+        :param share_id: The ID of the share
+        :param dict attrs: Attributes which will be used to create
+            a :class:`~openstack.shared_file_system.v2.
+            share_access_rules.ShareAccessRules`, comprised of the
+            properties on the ShareAccessRules class.
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            share_access_rules.ShareAccessRules`
+        """
+        base_path = "/shares/%s/action" % (share_id,)
+        return self._create(
+            _share_access_rule.ShareAccessRule, base_path=base_path, **attrs
+        )
+
+    def delete_access_rule(self, access_id, share_id, ignore_missing=True):
+        """Deletes an access rule
+
+        :param access_id: The id of the access rule to get
+        :param share_id: The ID of the share
+
+        :rtype: ``None``
+        """
+        res = self._get_resource(_share_access_rule.ShareAccessRule, access_id)
+        res.delete(self, share_id, ignore_missing=ignore_missing)
