@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+from __future__ import annotations
+
 from collections.abc import Mapping
 import hashlib
 import queue
@@ -17,6 +20,7 @@ import threading
 import time
 
 import keystoneauth1
+from keystoneauth1 import adapter as ks_adapter
 from keystoneauth1 import discover
 
 from openstack import _log
@@ -97,25 +101,56 @@ def get_string_format_keys(fmt_string, old_style=True):
         return keys
 
 
+def supports_version(
+    adapter: ks_adapter.Adapter,
+    version: str,
+    raise_exception: bool = False,
+) -> bool:
+    """Determine if the given adapter supports the given version.
+
+    Checks the version asserted by the service and ensures this matches the
+    provided version. ``version`` can be a major version or a major-minor
+    version
+
+    :param adapter: :class:`~keystoneauth1.adapter.Adapter` instance.
+    :param version: String containing the desired version.
+    :param raise_exception: Raise exception when requested version
+        is not supported by the server.
+    :returns: ``True`` if the service supports the version, else ``False``.
+    :raises: :class:`~openstack.exceptions.SDKException` when
+        ``raise_exception`` is ``True`` and requested version is not supported.
+    """
+    required = discover.normalize_version_number(version)
+
+    if discover.version_match(required, adapter.get_api_major_version()):
+        return True
+
+    if raise_exception:
+        raise exceptions.SDKException(
+            f'Required version {version} is not supported by the server'
+        )
+
+    return False
+
+
 def supports_microversion(adapter, microversion, raise_exception=False):
     """Determine if the given adapter supports the given microversion.
 
-    Checks the min and max microversion asserted by the service and checks to
-    make sure that ``min <= microversion <= max``. Current default microversion
-    is taken into consideration if set and verifies that ``microversion <=
-    default``.
+    Checks the min and max microversion asserted by the service and ensures
+    ``min <= microversion <= max``. If set, the current default microversion is
+    taken into consideration to ensure ``microversion <= default``.
 
     :param adapter: :class:`~keystoneauth1.adapter.Adapter` instance.
     :param str microversion: String containing the desired microversion.
     :param bool raise_exception: Raise exception when requested microversion
-        is not supported be the server side or is higher than the current
-        default microversion.
-    :returns: True if the service supports the microversion.
+        is not supported by the server or is higher than the current default
+        microversion.
+    :returns: True if the service supports the microversion, else False.
     :rtype: bool
-    :raises: :class:`~openstack.exceptions.SDKException` when requested
-        microversion is not supported.
+    :raises: :class:`~openstack.exceptions.SDKException` when
+        ``raise_exception`` is ``True`` and requested microversion is not
+        supported.
     """
-
     endpoint_data = adapter.get_endpoint_data()
     if (
         endpoint_data.min_microversion
@@ -367,7 +402,7 @@ class TinyDAG:
 
             except queue.Empty:
                 raise exceptions.SDKException(
-                    'Timeout waiting for ' 'cleanup task to complete'
+                    'Timeout waiting for cleanup task to complete'
                 )
         else:
             raise StopIteration
